/*
	Fractal - A program to calculate Newton fractals.
	Copyright (C) 2004 Johannes Bauer
	
	This file is part of Fractal.

    Fractal is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Fractal is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Fractal; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Johannes Bauer
	Mussinanstr. 140
	92318 Neumarkt/Opf.
	JohannesBauer@gmx.de
*/

#include <iostream>
#include <vector>
#include <algorithm>
#include <math.h>

#include "Fractal.hpp"
#include "CSolutions.hpp"

const double MinDistance = 0.01;
const double EPSILON = 0.0001;

CSolutions::CSolutions() {
}

double Distance(const cplx &Number1, const cplx &Number2) {
	return sqrt(pow(Number1.real() - Number2.real(), 2.0) + pow(Number1.imag() - Number2.imag(), 2.0));
}

int CSolutions::Get_Solution(const cplx &Number) {
	double MinimumDistance;
	int n;
	n=0;
	MinimumDistance=Distance(Solutions[0],Number);
	for (unsigned int i=1;i<this->Solutions.size();i++) {
		if (Distance(Solutions[i],Number)<MinimumDistance) {
			MinimumDistance=Distance(Solutions[i],Number);
			n=i;
		}
	}
	return n;
}

int CSolutions::Add_Solution(const cplx &Number) {
	for (unsigned int i=0;i<this->Solutions.size();i++) {
		if (Distance(Solutions[i],Number)<MinDistance) {
			return i;
		}
	}

	// We have a new solution!
	this->Solutions.push_back(Number);
	return this->Solutions.size()-1;
}

void CSolutions::Clear() {
	this->Solutions.clear();
}

void CSolutions::Show() {
	std::cout << "There exist " << this->Solutions.size() << " different solutions:" << std::endl;
	for (unsigned int i=0;i<this->Solutions.size();i++) {
		std::cout << (i+1) << ": " << this->Solutions[i] << std::endl;
	}
}

double absolute(double x);

int Compare(const cplx &First, const cplx &Second) {
	if (absolute(First.real()-Second.real()) < EPSILON) {
		if (absolute(First.imag()-Second.imag()) < EPSILON) {
			return 1;
		} else {
			if (First.imag()<Second.imag()) {
				return 1;
			} else {
				return 0;
			}
		}
	} else {
		if (First.real()<Second.real()) {
			return 1;
		} else {
			return 0;
		}
	}
}

void CSolutions::Sort() {
	std::sort(this->Solutions.begin(),this->Solutions.end(),Compare);
}

CSolutions::~CSolutions() {
}
